/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

import java.util.ArrayList;
import java.util.List;


/**
 * @author tbarros
 *
 * This visitor obtain the list of components of an expresion<p>
 * example:
 * <verbatim>
 * let exp = "A,B,C"
 * L = exp.dispatch(ComposantVisitor) = (ArrayList){A,B,C}
 * </verbatim>
 */
public class ComposantVisitor implements Visitor {
    ArrayList composants;

    public ComposantVisitor() {
        composants = new ArrayList();
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.OpExpr, java.lang.Object[])
     */
    public Object visit(OpExpr exp, Object[] params) {
        // TODO Auto-generated method stub
        if (",".equals(exp.getOp())) {
			exp.getSons()[0].dispatch(this,null);
			exp.getSons()[1].dispatch(this,null);
			return(null);
        }
        composants.add(exp);
        return(null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        // TODO Auto-generated method stub
		composants.add(exp);
        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
        // TODO Auto-generated method stub
		composants.add(exp);
        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
        // TODO Auto-generated method stub
		composants.add(exp);
        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
        // TODO Auto-generated method stub
		composants.add(exp);
        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
        // TODO Auto-generated method stub
		composants.add(exp);
        return null;
    }
	/**
	 * @return ArrayList
	 */
	public List getComposants() {
		return composants;
	}

}
