/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ package fr.inria.oasis.fc2.parser;

public class Declaration extends Type {
    protected int priority;

    public Declaration(String token, int classe, int[] domain, int codomain,
        int priority) {
        super(token, classe, domain, codomain);
        this.priority = priority;
    }

    /**
     * Constructor for Declaration.
     * @param token
     * @param classe
     * @param domain
     * @param codomain
     */
    public Declaration(String token, int classe, int[] domain, int codomain) {
        super(token, classe, domain, codomain);
        this.priority = OpExpr.LOW_PRIORITY;
    }

    public boolean equals(Object o) {
        if (o == null) {
            return (false);
        }

        if (o instanceof Declaration) {
            return (equals((Declaration) o));
        }

        return (false);
    }

    public boolean equals(Declaration decl) {
        if (decl == null) {
            return (false);
        }

        if (decl.getPriority() != priority) {
            return (false);
        }

        return (super.equals((Type) decl));
    }

    public void pp(java.io.PrintStream out, boolean s, String version) {
        String sclasse = Types.getMethodName(classe);
        String res = sclasse + " " + token + " (";

        for (int i = 0; i < domain.length; i++) {
            res = res + " " + Types.getTypeName(domain[i]);
        }

        if ((version.compareTo("\"1.1\"") >= 0) &&
                (classe == Types.OPERATOR_INFIX)) {
            out.println(res + ") -> " + Types.getTypeName(codomain) +
                " priority " + priority);
        } else {
            out.println(res + ") -> " + Types.getTypeName(codomain));
        }
    }

    /**
     * @return int
     */
    public int getPriority() {
        return priority;
    }

    /**
     * Sets the priority.
     * @param priority The priority to set
     */
    public void setPriority(int priority) {
        this.priority = priority;
    }
}
