/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ package fr.inria.oasis.fc2.parser;

public class Edge implements Cloneable {
    public Label label;
    public Expr targets;

    /* puisqu'il ssont cr??s par la methode ArrayList.toArray, nos
       tableaux de targets n'ont jamais d'elements nuls. On devrait donc
       pouvoir utiliser des int[] au lieu des Integer[]. Mais il faudrait
       faire ?a a la main... */
    public Edge(Label l, Expr t) {
        label = l;
        targets = t;
    }

    public Object clone() {
        Label newLabel = null;
        Expr newTargets = null;

        if (label != null) {
            newLabel = (Label) label.clone();
        }

        if (targets != null) {
            newTargets = (Expr) targets.clone();
        }

        return (new Edge(newLabel, newTargets));
    }

    /*
        public Edge(Label l, ArrayList a) {
            label = l;
            targets = (Integer[]) a.toArray(new Integer[0]);
        }
    */
    public String toString(int n) {
        String res = Types.getTypeName(Types.TYPE_EDGE) + " " + n + "\n" +
            label.toString() + " -> ";

        /*
                for (int i = 0; i < (targets.length - 1); i++) {
                    res = res + targets[i] + "+";
                }

                res = res + targets[targets.length - 1] + "\n";
        */
        return res + targets.toString() + "\n";
    }

    public void pp(java.io.PrintStream out, boolean s, int n) {
        String res = "";

        if (!s) {
            res = res + Types.getTypeName(Types.TYPE_EDGE, s) + n + " ";
        }

        res = res + label.toString(s) + " -> ";

        /*        for (int i = 0; i < (targets.length - 1); i++) {
                    res = res + targets[i] + "+";
                }

                res = res + targets[targets.length - 1];
                */
        out.println(res + targets.toString());
    }

    /**
     * @return Label
     */
    public Label getLabel() {
        return label;
    }

    /**
     * @return Integer[]
     */
    public Expr getTargets() {
        return targets;
    }

    /**
     * Sets the label.
     * @param label The label to set
     */
    public void setLabel(Label label) {
        this.label = label;
    }

    /**
     * Sets the targets.
     * @param targets The targets to set
     */
    public void setTargets(Expr targets) {
        this.targets = targets;
    }
}
