/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ package fr.inria.oasis.fc2.parser;

import java.io.InputStream;
import java.io.PrintStream;


public class Fc2Norm {
    /* default values: */
    static boolean debug_mode = false;
    static String format = "short";
    static boolean norm_net_types = false;
    static boolean norm_struct_table = false;
    static boolean norm_behav_table = false;
    static boolean norm_logic_table = false;
    static boolean norm_hook_table = false;
    static String infile = "-";
    static String outfile = "-";
    static InputStream in = System.in;
    static PrintStream out = System.out;
    static PrintStream err = System.err;

    static void usage() {
        err.println(
            "Usage: fc2norm [-?] [-d] [-l] [-s] [-t] [-u[type]] [infile [outfile]]\n" +
            "   Options:\n" + "      -? | -h : this message.\n" +
            "	-d : debug.\n" +
            "   Formats: the following are exclusive (default short):\n" +
            "	-l : long lexical form.\n" + "	-s : short lexical form.\n" +
            "   Normalization functions:\n" +
            "		In general, those are not exclusive, and will be\n" +
            "		applied in the order they appear:\n" +
            "	-t : Computes type of nets, registers them as hooks.\n" +
            "	-u[type] : Uniqueness of table entries\n" +
            "		Garanties that all fields of type [type] are\n" +
            "		tabulated, and that entries in the corresponding\n" +
            "		table are unique and pairwise distinct.\n" +
            "	   [type] is one of the letters in \"sblh\",\n" +
            "	   e.g. -ub means: tabulate uniquely behavs.\n" + "\n" +
            "	infile: input file, can be omitted if no outfile is present,\n" +
            "		or can be \"-\" for stdin.\n" +
            "	outfile: output file, can be omitted for stdout.\n");
        System.exit(-1);
    }

    static void decode_args(String[] args) {
        int i = 0;
opt: 
        for (i = 0; i < args.length; i++) {
            String arg = args[i];

            if ((arg.charAt(0) == '-') & (arg.length() > 1)) {
                switch (arg.charAt(1)) {
                case '?':usage();

                case 'd': {
                    debug_mode = true;

                    continue;
                }

                case 'l': {
                    format = "long";

                    continue;
                }

                case 's': {
                    format = "short";

                    continue;
                }

                case 't': {
                    norm_net_types = true;

                    continue;
                }

                case 'u': {
                    switch (arg.charAt(2)) {
                    case 's': {
                        norm_struct_table = true;

                        continue;
                    }

                    case 'b': {
                        norm_behav_table = true;

                        continue;
                    }

                    case 'l': {
                        norm_logic_table = true;

                        continue;
                    }

                    case 'h': {
                        norm_hook_table = true;

                        continue;
                    }

                    default:usage();
                    }
                }

                default:usage();
                }
            } else {
                break opt;
            }
        }

        /* there should only be 0, 1, or 2 filenames left,
           either "-" or legal pathnames */
        /* Input File arg */
        if (i < args.length) {
            if (!args[i].equals("-")) {
                infile = args[i];

                if ((new java.io.File(infile)).isFile()) {
                    try {
                        in = new java.io.FileInputStream(infile);

                        if (debug_mode) {
                            err.println("infile set to " + infile);
                        }
                    } catch (Exception e) {
                        err.println("Cannot open file " + infile);
                        System.exit(-1);
                    }
                } else {
                    err.println("not a filename, or not a file: " + infile);
                    usage();
                }
            }
        }

        i++;

        /* Output File arg */
        if (i < args.length) {
            if (args[i].equals("-")) {
                i++;
            } else {
                outfile = args[i];

                try {
                    out = new PrintStream(new java.io.FileOutputStream(outfile));
                } catch (Exception e) {
                    err.println("Cannot open output file " + outfile);
                    System.exit(-1);
                }

                i++;
            }
        }

        if (i < args.length) {
            err.println("Extra argument(s) ignored, starting at: " + args[i]);
        }
    }

    public static void main(String[] args) throws Exception {
        decode_args(args);

        Fc2Parser parser_obj = new Fc2Parser(new Yylex(in));
        parser_obj.set_trace_lex(debug_mode);
        parser_obj.set_trace_cup(debug_mode);
        parser_obj.parse();

        //out.println(normalize(parser_obj.abs_tree));
        Net res = normalize(parser_obj.abs_tree);
        res.pp(out, format.equals("short"));

        if (!outfile.equals("-")) {
            if (debug_mode) {
                err.println("Output written on file " + outfile);
            }
        }
    }

    static Net normalize(Net abs_tree) {
        if (norm_net_types | norm_struct_table | norm_behav_table |
                norm_logic_table | norm_hook_table) {
            err.println("Normalization not implemented");
        }

        return (abs_tree);
    }
}
