/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

public class Fc2Table {
    public int kind;
    public Expr[] t;

    public Fc2Table(int k) {
        kind = k;
    }

    public Fc2Table(int k, Expr[] table) {
        kind = k;
        t = table;
    }

    public void set(int i, Expr v) {
        t[i] = v;
    }

    public Expr ref(int i) {
        return t[i];
    }

    public int size() {
        return t.length;
    }

    public String toString() {
        String res = Types.getTypeName(kind) + " " + t.length + "\n";

        for (int i = 0; i < t.length; i++) {
            res = res + "  " + ": " + i + t[i] + "\n";
        }

        return res;
    }

    public void pp(java.io.PrintStream out, boolean s) {
        switch (kind) {
        case Types.TYPE_BEHAV:
            out.println(Types.getTypeName(Types.TABLE_BEHAVS, s) + t.length);

            break;

        case Types.TYPE_HOOK:
            out.println(Types.getTypeName(Types.TABLE_HOOKS, s) + t.length);

            break;

        case Types.TYPE_LOGIC:
            out.println(Types.getTypeName(Types.TABLE_LOGICS, s) + t.length);

            break;

        case Types.TYPE_STRUCT:
            out.println(Types.getTypeName(Types.TABLE_STRUCTS, s) + t.length);

            break;

        default:
            Fc2Norm.err.println("Fc2Norm: Internal error: bad table type: " +
                Types.getTypeName(kind));

            break;
        }

        for (int i = 0; i < t.length; i++) {
            if (s) {
                out.println(t[i]);
            } else {
                out.println(" :" + i + " " + t[i]);
            }
        }
    }

    public boolean verifyType(TypeCheckVisitor checkVisitor) {
        boolean result = true;

        for (int i = 0; i < t.length; i++) {
			result = ((Boolean)t[i].dispatch(checkVisitor,new Object[] { new Integer(kind)})).booleanValue();

            if (!result) {
                Utils.reportError("The expresion " + t[i] +
                    " didn't meet the typing rule T2 (see FC2 Reference Manual)");
            }
        }

        return (result);
    }
	/**
	 * @return int
	 */
	public int getKind() {
		return kind;
	}

	/**
	 * @return Expr[]
	 */
	public Expr[] getT() {
		return t;
	}

	/**
	 * Sets the kind.
	 * @param kind The kind to set
	 */
	public void setKind(int kind) {
		this.kind = kind;
	}

	/**
	 * Sets the t.
	 * @param t The t to set
	 */
	public void setT(Expr[] t) {
		this.t = t;
	}

}


/* ou met-on
       fun findTable ty tl = Lib.first (fn t => (kind t = ty)) tl
*/
