/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

import java.util.Vector;


public class Label implements Cloneable {
    public Expr struc; /* Attention: may be null */
    public Expr behav; /* idem */
    public Expr logic; /* idem */
    public Vector hooks;

    public Label(Expr s, Expr b, Expr l, Vector hs) {
        struc = s;
        behav = b;
        logic = l;
        hooks = hs;
    }

    public Label() {
        hooks = new Vector();
    }

    public Object clone() {
        Expr newStruc = null;
        Expr newBehav = null;
        Expr newLogic = null;
        Vector newHooks = null;

        if (struc != null) {
            newStruc = (Expr) struc.clone();
        }

        if (behav != null) {
            newBehav = (Expr) behav.clone();
        }

        if (logic != null) {
            newLogic = (Expr) logic.clone();
        }

        if (hooks != null) {
            newHooks = (Vector) hooks.clone();
        }

        return (new Label(newStruc, newBehav, newLogic, newHooks));
    }

    public Expr get_struc() {
        return struc;
    }

    public Expr get_behav() {
        return behav;
    }

    public Expr get_logic() {
        return logic;
    }

    public Vector get_hooks() {
        return hooks;
    }

    public Label set_struc(Expr s) {
        struc = s;

        return this;
    }

    public Label set_behav(Expr b) {
        behav = b;

        return this;
    }

    public Label set_logic(Expr l) {
        logic = l;

        return this;
    }

    public Label add_hook(Expr h) {
        hooks.add(h);

        return this;
    }

    public String toString(boolean s) {
        String r = "";

        if (hooks != null) {
            for (int i = 0; i < hooks.size(); i++) {
                r = r + Types.getTypeName(Types.TYPE_HOOK, s) + hooks.get(i);
            }
        }

        if (struc != null) {
            r = r + Types.getTypeName(Types.TYPE_STRUCT, s) + struc;
        }

        if (behav != null) {
            r = r + Types.getTypeName(Types.TYPE_BEHAV, s) + behav;
        }

        if (logic != null) {
            r = r + Types.getTypeName(Types.TYPE_LOGIC, s) + logic;
        }

        return r;
    }

    public void pp(java.io.PrintStream out, boolean s) {
        for (int i = 0; (hooks != null) && (i < hooks.size()); i++) {
            out.println(Types.getTypeName(Types.TYPE_HOOK, s) + hooks.get(i));
        }

        if (struc != null) {
            out.println(Types.getTypeName(Types.TYPE_STRUCT, s) + struc);
        }

        if (behav != null) {
            out.println(Types.getTypeName(Types.TYPE_BEHAV, s) + behav);
        }

        if (logic != null) {
            out.println(Types.getTypeName(Types.TYPE_LOGIC, s) + logic);
        }
    }

    public boolean verifyType(TypeCheckVisitor checkVisitor) {
        boolean result = true;

        // Rule T1
        if (behav != null) {
            result = ((Boolean) behav.dispatch(checkVisitor,
                    new Object[] { new Integer(Types.TYPE_BEHAV) })).booleanValue();
        }

        if (!result) {
            Utils.reportError("The expresion " + behav +
                " didn't meet the typing rule T1 (see FC2 Reference Manual)");
        }

        if (logic != null) {
            result = ((Boolean) logic.dispatch(checkVisitor,
                    new Object[] { new Integer(Types.TYPE_LOGIC) })).booleanValue();

            if (!result) {
                Utils.reportError("The expresion " + logic +
                    " didn't meet the typing rule T1 (see FC2 Reference Manual)");
            }
        }

        if (struc != null) {
            result = ((Boolean) struc.dispatch(checkVisitor,
                    new Object[] { new Integer(Types.TYPE_STRUCT) })).booleanValue();

            if (!result) {
                Utils.reportError("The expresion " + struc +
                    " didn't meet the typing rule T1 (see FC2 Reference Manual)");
            }
        }

        return (result);
    }
}
