/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

import java.util.ArrayList;


public class NetNode {
    public Label label;
    public Fc2Table[] localTables;
    public Vertex[] vertice;
    public int num;

    public NetNode(Label l, Fc2Table[] t, Vertex[] v, int n) {
        label = l;
        localTables = t;
        vertice = v;
        num = n;
    }

    public NetNode(Label l, ArrayList al, Vertex[] v, int n) {
        label = l;
        localTables = (Fc2Table[]) al.toArray(new Fc2Table[0]);
        vertice = v;
        num = n;
    }

    public void pp(java.io.PrintStream out, boolean s) {
        for (int i = 0; i < localTables.length; i++) {
            localTables[i].pp(out, s);
        }

        if (label != null) {
            label.pp(out, s);
        }

        out.println(Types.getTypeName(Types.TABLE_VERTICE, s) + vertice.length);

        for (int i = 0; i < vertice.length; i++) {
            vertice[i].pp(out, s, i);
        }
    }

    /**
     * Returns the label.
     * @return Label
     */
    public Label getLabel() {
        return label;
    }

    /**
     * Returns the localTables.
     * @return Fc2Table[]
     */
    public Fc2Table[] getLocalTables() {
        return localTables;
    }

    /**
     * Returns the num.
     * @return int
     */
    public int getNum() {
        return num;
    }

    /**
     * Returns the vertice.
     * @return Vertex[]
     */
    public Vertex[] getVertice() {
        return vertice;
    }

    /**
     * Sets the label.
     * @param label The label to set
     */
    public void setLabel(Label label) {
        this.label = label;
    }

    /**
     * Sets the localTables.
     * @param localTables The localTables to set
     */
    public void setLocalTables(Fc2Table[] localTables) {
        this.localTables = localTables;
    }

    /**
     * Sets the num.
     * @param num The num to set
     */
    public void setNum(int num) {
        this.num = num;
    }

    /**
     * Sets the vertice.
     * @param vertice The vertice to set
     */
    public void setVertice(Vertex[] vertice) {
        this.vertice = vertice;
    }
}
