/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;


/**
 * @author tbarros
 *
 */
public class Type {
    protected int classe;
    protected String token;
    protected int[] domain;
    protected int codomain;
    protected boolean unboundedDomain = false;
    protected Expr value = null;

    public Type(String token, int classe, int[] domain, int codomain,
        boolean unbounded) {
        this(token, classe, domain, codomain);
        this.unboundedDomain = unbounded;
    }

    public Type(String token, int classe, int[] domain, int codomain) {
        this.token = token;
        this.classe = classe;
        this.domain = domain;
        this.codomain = codomain;
    }

	public boolean equals(Object o) {
		if (o == null) return (false);
		if (o instanceof Type) {
			return (equals((Type) o));
		}
		return(false);
	}
	
    public boolean equals(Type type) {
        if (type == null) {
            return (false);
        }

        if ((type.getClasse() != classe) || (type.getCodomain() != codomain)) {
            return (false);
        }

        if (!(((type.getToken() == null) && (token == null)) ||
                ((token != null) && token.equals(type.getToken())))) {
            return (false);
        }

        if (type.getDomain().length != domain.length) {
            return (false);
        }

        for (int i = 0; i < domain.length; i++) {
            if (domain[i] != type.getDomain()[i]) {
                return (false);
            }
        }

        if (!(((type.getValue() == null) && (value == null)) ||
                ((value != null) && value.equals(type.getValue())))) {
            return (false);
        }

        return true;
    }

    /**
     * Returns the classe.
     * @return int
     */
    public int getClasse() {
        return classe;
    }

    /**
     * Returns the token.
     * @return String
     */
    public String getToken() {
        return token;
    }

    /**
     * Sets the classe.
     * @param classe The classe to set
     */
    public void setClasse(int classe) {
        this.classe = classe;
    }

    /**
     * Sets the token.
     * @param token The token to set
     */
    public void setToken(String token) {
        this.token = token;
    }

    /**
     * Returns the codomain.
     * @return int
     */
    public int getCodomain() {
        return codomain;
    }

    /**
     * Returns the domain.
     * @return int[]
     */
    public int[] getDomain() {
        return domain;
    }

    /**
     * Sets the codomain.
     * @param codomain The codomain to set
     */
    public void setCodomain(int codomain) {
        this.codomain = codomain;
    }

    /**
     * Sets the domain.
     * @param domain The domain to set
     */
    public void setDomain(int[] domain) {
        this.domain = domain;
    }

    /**
     * Returns the unboundedDomain.
     * @return boolean
     */
    public boolean isUnboundedDomain() {
        return unboundedDomain;
    }

    /**
     * Sets the unboundedDomain.
     * @param unboundedDomain The unboundedDomain to set
     */
    public void setUnboundedDomain(boolean unboundedDomain) {
        this.unboundedDomain = unboundedDomain;
    }

    /**
     * @return Expr
     */
    public Expr getValue() {
        return value;
    }

    /**
     * Sets the value.
     * @param value The value to set
     */
    public void setValue(Expr value) {
        this.value = value;
    }
}
