/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2.parser;

import java.util.ArrayList;


/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class TypeCheckVisitor implements Visitor {
    Types simbols;

    public TypeCheckVisitor(Types simbols) {
        this.simbols = simbols;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.OpExpr, java.lang.Object[])
     */
    public Object visit(OpExpr exp, Object[] params) {
        // TODO Auto-generated method stub
        boolean result = true;
        Integer type = ((Integer) params[0]);

        Type refType = simbols.getType(exp.getOp());

        if (refType == null) {
            Utils.reportError("Checking Label Types: The operator " +
                exp.getOp() + " is not defined");
            result = false;
        }

        if ((refType.getCodomain() != Types.TYPE_ANY) &&
                (refType.getCodomain() != type.intValue())) {
            Utils.reportError("Checking Types: The operator " + exp.getOp() +
                " return the type " + Types.getTypeName(refType.getCodomain()) +
                " and it should returns the type " +
                Types.getTypeName(type.intValue()));
            result = false;
        }

        switch (refType.getClasse()) {
        case Types.OPERATOR_INFIX:

            if (refType.getDomain()[0] == Types.TYPE_ANY) {
                result = result &&
                    ((Boolean) exp.getSons()[0].dispatch(this,
                        new Object[] { type })).booleanValue();
            } else {
                result = result &&
                    ((Boolean) exp.getSons()[0].dispatch(this,
                        new Object[] { new Integer(refType.getDomain()[0]) })).booleanValue();
            }

            if (refType.getDomain()[1] == Types.TYPE_ANY) {
                return (new Boolean(result &&
                    ((Boolean) exp.getSons()[1].dispatch(this,
                        new Object[] { type })).booleanValue()));
            } else {
                return (new Boolean(result &&
                    ((Boolean) exp.getSons()[1].dispatch(this,
                        new Object[] { new Integer(refType.getDomain()[1]) })).booleanValue()));
            }

        case Types.OPERATOR_PREFIX:

ComposantVisitor spar = new ComposantVisitor();
            exp.getSons()[0].dispatch(spar, null);

            ArrayList epar = (ArrayList)spar.getComposants();

            if (epar.size() != refType.getDomain().length) {
                Utils.reportError(
                    "Wrong number of parameters for the function " +
                    exp.getOp());

                return (new Boolean(false));
            }

            for (int i = 0; i < epar.size(); i++) {
                if (refType.getDomain()[i] == Types.TYPE_ANY) {
                    result = result &&
                        ((Boolean) ((Expr)epar.get(i)).dispatch(this, new Object[] { type })).booleanValue();
                } else {
                    result = result &&
                        ((Boolean) ((Expr)epar.get(i)).dispatch(this,
                            new Object[] { new Integer(refType.getDomain()[i]) })).booleanValue();
                }
            }

            return (new Boolean(result));

        case Types.OPERATOR_UNARY:

            if (refType.getDomain()[0] == Types.TYPE_ANY) {
                return (new Boolean(result &&
                    ((Boolean) exp.getSons()[0].dispatch(this,
                        new Object[] { type })).booleanValue()));
            } else {
                return (new Boolean(result &&
                    ((Boolean) exp.getSons()[0].dispatch(this,
                        new Object[] { new Integer(refType.getDomain()[0]) })).booleanValue()));
            }

        default:
            return (new Boolean(result));
        }
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        // TODO Auto-generated method stub
        Integer type = ((Integer) params[0]);

        if (type.intValue() == Types.TYPE_INT) {
            Utils.reportError("Type Checking: the expresion " + exp.getValue() +
                " it should be of type " + Types.getTypeName(type.intValue()));

            return (new Boolean(false));
        }

        return (new Boolean(true));
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
        // TODO Auto-generated method stub
        return new Boolean(true);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
        // TODO Auto-generated method stub
        return new Boolean(true);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
        // TODO Auto-generated method stub
        Integer type = ((Integer) params[0]);

        if (type.intValue() == Types.TYPE_INT) {
            Utils.reportError("Type Checking: the expresion " + exp.getValue() +
                " it should be of type " + Types.getTypeName(type.intValue()));

            return (new Boolean(false));
        }

        return (new Boolean(true));
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
        // TODO Auto-generated method stub
        Integer type = ((Integer) params[0]);

        if (type.intValue() == Types.TYPE_INT) {
            Utils.reportError("Type Checking: the expresion " + exp.getValue() +
                " it should be of type " + Types.getTypeName(type.intValue()));

            return (new Boolean(false));
        }

        return (new Boolean(true));
    }
}
