/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;


/**
 * @author tbarros
 *
 */
public class Types {
    public static final int OPERATOR_CONSTANT = 0x01;
    public static final int OPERATOR_UNARY = 0x02;
    public static final int OPERATOR_INFIX = 0x03;
    public static final int OPERATOR_PREFIX = 0x04;

    /*************/
    public static final int TYPE_BEHAV = 0x21;
    public static final int TYPE_STRUCT = 0x22;
    public static final int TYPE_LOGIC = 0x23;
    public static final int TYPE_HOOK = 0x24;
    public static final int TYPE_INT = 0x25;
    public static final int TYPE_ANY = 0x26;
    public static final int TYPE_NET = 0x27;
    public static final int TYPE_VERTEX = 0x28;
    public static final int TYPE_EDGE = 0x29;
    public static final int TYPE_ERROR = 0x2A;

    /*************/
    public static final int TABLE_EDGES = 0x41;
    public static final int TABLE_BEHAVS = 0x42;
    public static final int TABLE_HOOKS = 0x43;
    public static final int TABLE_NETS = 0x44;
    public static final int TABLE_STRUCTS = 0x45;
    public static final int TABLE_VERTICE = 0x46;
    public static final int TABLE_LOGICS = 0x47;

    /*************/
    public static final int LIST_DECLARATIONS = 0x61;

    /*************/
    protected HashMap types;

    public Types() {
        types = new HashMap();

        //  addDefaultTypes();
    }

    public boolean equals(Object o) {
        if (o == null) {
            return (false);
        }

        if (o instanceof Types) {
            return (equals((Types) o));
        }

        return (false);
    }

    public boolean equals(Types ref) {
        if (ref == null) {
            return (false);
        }

        if ((ref.getTypes() == null) && (types == null)) {
            return (true);
        }

        if ((ref.getTypes() == null) || (types == null)) {
            return (false);
        }

        Set kset = types.keySet();

        if (!kset.equals(ref.getTypes().keySet())) {
            return (false);
        }

        for (Iterator i = kset.iterator(); i.hasNext();) {
            Object next = i.next();

            if (!types.get(next).equals(ref.getTypes().get(next))) {
                return (false);
            }
        }

        return (true);
    }

    public static boolean isPredefined(OpExpr exp) {
        if ("t".equals(exp.getOp()) || "q".equals(exp.getOp()) ||
                "_".equals(exp.getOp())) {
            return true;
        }

        return false;
    }

    public static String getTypeName(int type) {
        return getTypeName(type, false);
    }

    public static String getTypeName(int type, boolean isShort) {
        switch (type) {
        case Types.TYPE_EDGE:

            if (isShort) {
                return new String("e");
            } else {
                return new String("edge");
            }

        case Types.TABLE_EDGES:

            if (isShort) {
                return new String("E");
            } else {
                return new String("edges ");
            }

        case Types.TYPE_NET:

            if (isShort) {
                return new String("n");
            } else {
                return new String("net ");
            }

        case Types.TABLE_NETS:

            if (isShort) {
                return new String("N");
            } else {
                return new String("nets ");
            }

        case Types.TYPE_VERTEX:

            if (isShort) {
                return new String("v");
            } else {
                return new String("vertex");
            }

        case Types.TABLE_VERTICE:

            if (isShort) {
                return new String("V");
            } else {
                return new String("vertice ");
            }

        case Types.TYPE_BEHAV:

            if (isShort) {
                return new String("b ");
            } else {
                return new String("behav ");
            }

        case Types.TABLE_BEHAVS:

            if (isShort) {
                return new String("B");
            } else {
                return new String("behavs ");
            }

        case Types.TYPE_STRUCT:

            if (isShort) {
                return new String("s ");
            } else {
                return new String("struct ");
            }

        case Types.TABLE_STRUCTS:

            if (isShort) {
                return new String("S");
            } else {
                return new String("structs ");
            }

        case Types.TYPE_LOGIC:

            if (isShort) {
                return new String("l ");
            } else {
                return new String("logic ");
            }

        case Types.TABLE_LOGICS:

            if (isShort) {
                return new String("L");
            } else {
                return new String("logics ");
            }

        case Types.TYPE_HOOK:

            if (isShort) {
                return new String("h ");
            } else {
                return new String("hook ");
            }

        case Types.TABLE_HOOKS:

            if (isShort) {
                return new String("H");
            } else {
                return new String("hooks ");
            }

        /****************/
        case Types.LIST_DECLARATIONS:

            if (isShort) {
                return new String("D");
            } else {
                return new String("declarations");
            }

        /****************/
        case Types.TYPE_INT:
            return new String("int");

        case Types.TYPE_ANY:
            return new String("any");

        case Types.TYPE_ERROR:
            return new String("error");

        /****************/
        default:
            return new String("** Unknow Type **");
        }
    }

    public static String getMethodName(int type) {
        switch (type) {
        case Types.OPERATOR_CONSTANT:
            return new String("constant");

        case Types.OPERATOR_INFIX:
            return new String("infix");

        case Types.OPERATOR_PREFIX:
            return new String("prefix");

        case Types.OPERATOR_UNARY:
            return new String("unary");

        default:
            return new String("** Unknow Type **");
        }
    }

    public void addDefaultTypes() {
        addType(new Type("t", Types.OPERATOR_CONSTANT, new int[] {  },
                Types.TYPE_BEHAV));
        addType(new Type("q", Types.OPERATOR_CONSTANT, new int[] {  },
                Types.TYPE_BEHAV));
        addType(new Type("_", Types.OPERATOR_CONSTANT, new int[] {  },
                Types.TYPE_ANY));
        addType(new Type("!", Types.OPERATOR_UNARY,
                new int[] { Types.TYPE_ANY }, Types.TYPE_ANY));
        addType(new Type("?", Types.OPERATOR_UNARY,
                new int[] { Types.TYPE_ANY }, Types.TYPE_ANY));
        addType(new Type("~", Types.OPERATOR_UNARY,
                new int[] { Types.TYPE_ANY }, Types.TYPE_ANY));
        addType(new Type("#", Types.OPERATOR_UNARY,
                new int[] { Types.TYPE_ANY }, Types.TYPE_ANY));
        addType(new Type("^", Types.OPERATOR_INFIX,
                new int[] { Types.TYPE_ANY, Types.TYPE_INT }, Types.TYPE_ANY,
                true));
        addType(new Type("+", Types.OPERATOR_INFIX,
                new int[] { Types.TYPE_ANY, Types.TYPE_ANY }, Types.TYPE_ANY,
                true));
        addType(new Type("<", Types.OPERATOR_INFIX,
                new int[] { Types.TYPE_ANY, Types.TYPE_ANY }, Types.TYPE_ANY,
                true));
        addType(new Type(">", Types.OPERATOR_INFIX,
                new int[] { Types.TYPE_ANY, Types.TYPE_ANY }, Types.TYPE_ANY,
                true));
        addType(new Type(",", Types.OPERATOR_INFIX,
                new int[] { Types.TYPE_ANY, Types.TYPE_ANY }, Types.TYPE_ANY,
                true));
        addType(new Type(";", Types.OPERATOR_INFIX,
                new int[] { Types.TYPE_ANY, Types.TYPE_ANY }, Types.TYPE_ANY,
                true));
        addType(new Type(".", Types.OPERATOR_INFIX,
                new int[] { Types.TYPE_ANY, Types.TYPE_ANY }, Types.TYPE_ANY,
                true));
    }

    public void addType(Type type) {
        this.types.put(type.getToken(), type);
    }

    public Type getType(String token) {
        return ((Type) types.get(token));
    }

    /**
     * @return HashMap
     */
    public HashMap getTypes() {
        return types;
    }

    /**
     * Sets the types.
     * @param types The types to set
     */
    public void setTypes(HashMap types) {
        this.types = types;
    }
}
