/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

public class Vertex implements Cloneable {
    public Label label;
    public Edge[] edges;

    public Vertex(Label l, Edge[] e) {
        label = l;
        edges = e;
    }

    public Object clone() {
        Label newLabel = null;
        Edge[] newEdges = new Edge[edges.length];

        for (int i = 0; i < edges.length; i++) {
            if (edges[i] != null) {
                newEdges[i] = (Edge) edges[i].clone();
            } else {
                newEdges[i] = null;
            }
        }

        if (label != null) {
            newLabel = (Label) label.clone();
        }

        return (new Vertex(newLabel, newEdges));
    }

    public String toString(int n) {
        String res = Types.getTypeName(Types.TYPE_VERTEX) + " " + n + "\n" +
            label.toString() + "\n";
        res = res + Types.getTypeName(Types.TABLE_EDGES) + " " + edges.length +
            "\n";

        for (int i = 0; i < edges.length; i++) {
            res = res + edges[i].toString(i);
        }

        return res;
    }

    public void pp(java.io.PrintStream out, boolean s, int n) {
        out.println(Types.getTypeName(Types.TYPE_VERTEX, s) + n);
        label.pp(out, s);
        out.println(Types.getTypeName(Types.TABLE_EDGES, s) + edges.length);

        for (int i = 0; i < edges.length; i++) {
            edges[i].pp(out, s, i);
        }
    }

    /**
     * @return Edge[]
     */
    public Edge[] getEdges() {
        return edges;
    }

    /**
     * @return Label
     */
    public Label getLabel() {
        return label;
    }

    /**
     * Sets the edges.
     * @param edges The edges to set
     */
    public void setEdges(Edge[] edges) {
        this.edges = edges;
    }

    /**
     * Sets the label.
     * @param label The label to set
     */
    public void setLabel(Label label) {
        this.label = label;
    }
}
