/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2;

import java.io.PrintStream;
import java.util.HashMap;

import fr.inria.oasis.fc2.parser.Bquote;
import fr.inria.oasis.fc2.parser.Fc2String;
import fr.inria.oasis.fc2.parser.GlobalRef;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Visitor;


/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class BehavVisitor implements Visitor {
    protected static final MathSolverVisitor mathSolverVisitor = new MathSolverVisitor();
    protected boolean started = false;
    protected PrintStream netOut;
    protected PrintStream infoOut;

    public BehavVisitor(PrintStream netOut, PrintStream infoOut) {
        this.netOut = netOut;
        this.infoOut = infoOut;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.OpExpr, java.lang.Object[])
     */
    public Object visit(OpExpr exp, Object[] params) {
        HashMap references = (HashMap) params[1];

        // TODO Auto-generated method stub
        if ("?".equals(exp.getOp()) || "!".equals(exp.getOp())) {
            Reference ref = (Reference) references.get(exp.getSons()[0].dispatch(
                        mathSolverVisitor, params));

            if (ref == null) {
                Utils.reportError("There is not a valid reference for " +
                    exp.getSons()[0] + " in " + exp);

                return (null);
            }

            for (int i = 0; i < ref.size(); i++) {
                if (started) {
                    netOut.print("+" + exp.getOp() + (int) (ref.offset + i));
                } else {
                    netOut.print(exp.getOp() + (int) (ref.offset + i));
                    started = true;
                }
            }

            return (null);
        }

        if ("+".equals(exp.getOp())) {
            exp.getSons()[0].dispatch(this, params);
            exp.getSons()[1].dispatch(this, params);

            return (null);
        }

        Utils.reportError("The expression " + exp +
            " is not a valid net reference");

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        HashMap references = (HashMap) params[1];
        Reference ref = (Reference) references.get(exp.dispatch(
                    mathSolverVisitor, params));

        if (ref == null) {
            Utils.reportError("There is not a valid reference for " + exp +
                " in " + exp);

            return (null);
        }

        for (int i = 0; i < ref.size(); i++) {
            if (started) {
                netOut.print("+" + (int) (ref.offset + i));
            } else {
                netOut.print((int) (ref.offset + i));
                started = true;
            }
        }

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
        HashMap references = (HashMap) params[1];
        Reference ref = (Reference) references.get(exp.dispatch(
                    mathSolverVisitor, params));

        if (ref == null) {
            Utils.reportError("There is not a valid reference for " + exp +
                " in " + exp);

            return (null);
        }

        for (int i = 0; i < ref.size(); i++) {
            if (started) {
                netOut.print("+" + (int) (ref.offset + i));
            } else {
                netOut.print((int) (ref.offset + i));
                started = true;
            }
        }

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
        HashMap references = (HashMap) params[1];
        Reference ref = (Reference) references.get(exp.dispatch(
                    mathSolverVisitor, params));

        if (ref == null) {
            Utils.reportError("There is not a valid reference for " + exp +
                " in " + exp);

            return (null);
        }

        for (int i = 0; i < ref.size(); i++) {
            if (started) {
                netOut.print("+" + (int) (ref.offset + i));
            } else {
                netOut.print((int) (ref.offset + i));
                started = true;
            }
        }

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
        HashMap references = (HashMap) params[1];
        Reference ref = (Reference) references.get(exp.dispatch(
                    mathSolverVisitor, params));

        if (ref == null) {
            Utils.reportError("There is not a valid reference for " + exp +
                " in " + exp);

            return (null);
        }

        for (int i = 0; i < ref.size(); i++) {
            if (started) {
                netOut.print("+" + (int) (ref.offset + i));
            } else {
                netOut.print((int) (ref.offset + i));
                started = true;
            }
        }

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
        HashMap references = (HashMap) params[1];
        Reference ref = (Reference) references.get(exp.dispatch(
                    mathSolverVisitor, params));

        if (ref == null) {
            Utils.reportError("There is not a valid reference for " + exp +
                " in " + exp);

            return (null);
        }

        for (int i = 0; i < ref.size(); i++) {
            if (started) {
                netOut.print("+" + (int) (ref.offset + i));
            } else {
                netOut.print((int) (ref.offset + i));
                started = true;
            }
        }

        return (null);
    }
}
