/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.List;

import fr.inria.oasis.fc2.parser.ComposantVisitor;
import fr.inria.oasis.fc2.parser.Edge;
import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Types;


/**
 * @author bbarros
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class EdgeExpandPrintStrategy extends StrategyAbstractImplementation {
    protected Edge edge;
    public int offset;
    public PrintStream netOut;
    public PrintStream infoOut;
    protected HashMap behavsReferences;
    protected Reference[] vertexReferences;
    protected int vectorSize;
    protected HashMap netReferences;
    protected HashMap[] allNetsBehavsTable;

    public EdgeExpandPrintStrategy(Edge edge, int offset, PrintStream netOut,
        PrintStream infoOut, HashMap behavsReferences,
        Reference[] vertexReferences, int vectorSize, HashMap netReferences,
        HashMap[] allNetsBehavsTable) {
        this.offset = offset;
        this.edge = edge;
        this.netOut = netOut;
        this.infoOut = infoOut;
        this.behavsReferences = behavsReferences;
        this.vertexReferences = vertexReferences;
        this.vectorSize = vectorSize;
        this.netReferences = netReferences;
        this.allNetsBehavsTable = allNetsBehavsTable;
    }

    protected void removeHooks(List hooks) {
        int i = 0;

        while (i < hooks.size()) {
            try {
                OpExpr exp = (OpExpr) hooks.get(i);

                if (("when".equals(exp.getOp())) || ("=".equals(exp.getOp()))) {
                    hooks.remove(i);

                    continue;
                }
            } catch (ClassCastException e) {
            }

            i++;
        }
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.StrategyAbstractImplementation#execute(fr.inria.oasis.fc2.parser.Mapping)
     */
    public void execute(Mapping map) {
        // TODO Auto-generated method stub
        Edge newEdge = (Edge) edge.clone();
        removeHooks(newEdge.getLabel().get_hooks());

        TargetVisitor targetv = new TargetVisitor(vertexReferences);
        Expr newTarget = (Expr) edge.getTargets().dispatch(targetv,
                new Object[] { map });

        if (edge.getLabel().get_behav() != null) {
            if (edge.getLabel().get_behav() instanceof OpExpr) {
                if ("<".equals(((OpExpr) edge.getLabel().get_behav()).getOp())) {
                    OpExpr exp = (OpExpr) edge.getLabel().get_behav().clone();

                    SyncVector vec = new SyncVector(vectorSize);

                    ReplaceBehavEdgeVisitor repv = new ReplaceBehavEdgeVisitor(behavsReferences);
                    vec.setAffectation((Expr) exp.getSons()[0].dispatch(repv,
                            new Object[] { (map) }));

                    ComposantVisitor compv = new ComposantVisitor();
                    exp.getSons()[1].dispatch(compv, null);

                    List composants = compv.getComposants();

                    ReplaceNetVisitor rnetv = new ReplaceNetVisitor(netReferences,
                            allNetsBehavsTable);

                    for (int j = 0; j < composants.size(); j++) {
                        ((Expr) composants.get(j)).dispatch(rnetv,
                            new Object[] { map });
                        vec.setBehavAt(rnetv.getBehav(), rnetv.getNetPlace());
                    }

                    //I go much more faster if I print the edges inmediately
                    netOut.print(Types.getTypeName(Types.TYPE_EDGE) + offset +
                        " ");

                    if (newEdge.getLabel().get_hooks() != null) {
                        for (int i = 0;
                                i < newEdge.getLabel().get_hooks().size();
                                i++) {
                            netOut.print(Types.getTypeName(Types.TYPE_HOOK,
                                    false) +
                                newEdge.getLabel().get_hooks().get(i));
                        }
                    }

                    if (newEdge.getLabel().get_struc() != null) {
                        netOut.print(Types.getTypeName(Types.TYPE_STRUCT, false) +
                            newEdge.getLabel().get_struc());
                    }

                    if (newEdge.getLabel().get_behav() != null) {
                        netOut.print(Types.getTypeName(Types.TYPE_BEHAV) + " ");
                        vec.pp(netOut);
                    }

                    if (newEdge.getLabel().get_logic() != null) {
                        netOut.print(Types.getTypeName(Types.TYPE_LOGIC, false) +
                            newEdge.getLabel().get_logic());
                    }

                    netOut.println(" -> " + newTarget);

                    offset++;

                    return;
                }
            }
                Expr oldBehav = (Expr) edge.getLabel().get_behav().clone();

                ReplaceBehavEdgeVisitor repv = new ReplaceBehavEdgeVisitor(behavsReferences);

                oldBehav = (Expr) oldBehav.dispatch(repv, new Object[] { map });

                newEdge.setTargets(newTarget);
                newEdge.getLabel().set_behav((Expr) oldBehav);

                newEdge.pp(netOut, false, offset);
                offset++;
            
        }
    }

    /**
     * @return
     */
    public int getOffset() {
        return offset;
    }
}
