/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import jargs.gnu.CmdLineParser;

import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import fr.inria.oasis.fc2.parser.ComposantVisitor;
import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.Fc2Parser;
import fr.inria.oasis.fc2.parser.Fc2Table;
import fr.inria.oasis.fc2.parser.Label;
import fr.inria.oasis.fc2.parser.Net;
import fr.inria.oasis.fc2.parser.NetNode;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Types;
import fr.inria.oasis.fc2.parser.Vertex;


/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class FC2Instantiate {
    protected static final MergeInstancesVisitor mergeInstancesVisitor = new MergeInstancesVisitor();
    protected static final MathSolverVisitor mathSolverVisitor = new MathSolverVisitor();

    // protected static final BehavVisitor behavVisitor = new BehavVisitor();
    Net decl;
    Net[] inst;
    protected Mapping globalValues;
    public PrintStream netOut;
    public PrintStream infoOut;

    public FC2Instantiate(Net decl, Net[] inst, PrintStream netOut,
        PrintStream infoOut) {
        this.decl = decl;
        this.inst = inst;
        this.infoOut = infoOut;
        this.netOut = netOut;
    }

    public boolean verifySimbolsTables(Net decl, Net inst) {
        if (decl.getSymbolTable().equals(inst.getSymbolTable())) {
            return (true);
        }

        System.err.println(
            "The simbol table between the declaration and instanciation fc2 specification are not the same. Please check that the declaration section in both fc2 specifications");

        return (false);
    }

    protected void expandAndPrintTable(Mapping symbolTable, Fc2Table behav,
        Map refTable, int type) {
        Expr[] exps = behav.getT();
        ReplaceTableVisitor repv = new ReplaceTableVisitor(new PreEvalTableStrategy(
                    netOut, infoOut));

        for (int j = 0; j < exps.length; j++) {
            exps[j].dispatch(repv, new Object[] { symbolTable });
        }

        netOut.println(Types.getTypeName(type) + repv.getOffset());

        repv = new ReplaceTableVisitor(new TableStrategy(netOut, infoOut));

        for (int j = 0; j < exps.length; j++) {
            Reference ref = (Reference) exps[j].dispatch(repv,
                    new Object[] { symbolTable });
            refTable.put(new Integer(j), ref);
        }
    }

    protected int expandStruct(Mapping symbolTable, Expr struct,
        HashMap netReferences) {
        try {
            OpExpr exp = (OpExpr) struct.clone();

            if (!"<".equals(exp.getOp())) {
            	
				ReplaceTableVisitor repv = new ReplaceTableVisitor(new ExprStrategy(netOut, infoOut));

					exp.dispatch(repv,
							new Object[] { symbolTable });

                return (0);
            }

            NetVisitor netv = new NetVisitor(netReferences, netOut, infoOut);

            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[1].dispatch(compv, null);

            List composants = compv.getComposants();

            netOut.print(((OpExpr) struct).getSons()[0] +
                ((OpExpr) struct).getOp());

            for (int i = 0; i < composants.size(); i++) {
                ((Expr) composants.get(i)).dispatch(netv,
                    new Object[] { symbolTable });
            }

            netOut.println();

            return (netv.getOffset());
        } catch (ClassCastException e) {
            netOut.print(struct + "\n");

            return (0);
        }
    }

    public Expr getVariables(List hooks) {
        ArrayList variables = new ArrayList();

        for (int i = 0; i < hooks.size(); i++) {
            Object o = hooks.get(i);

            if (o instanceof OpExpr) {
                OpExpr exp = (OpExpr) o;

                if ("=".equals(exp.getOp())) {
                    variables.add(exp.getSons()[0]);
                }
            }
        }

        if (variables.size() == 0) {
            return (null);
        }

        Expr exp = (Expr) variables.get(variables.size() - 1);

        for (int i = variables.size() - 2; i >= 0; i--) {
            exp = new OpExpr(",", new Expr[] { (Expr) variables.get(i), exp },
                    Types.OPERATOR_INFIX, 30);
        }

        return exp;
    }

    public void expand() {
        infoOut.println("Getting instantiation values");

        GlobalAssignVisitor globalv = new GlobalAssignVisitor();

        NetNode[] nets = decl.get_nets();

        for (int k = 0; k < inst.length; k++) {
            Vector gHooks = inst[k].get_nets()[0].getLabel().get_hooks();

            for (int i = 0; i < gHooks.size(); i++) {
                ((Expr) gHooks.get(i)).dispatch(globalv, null);
            }
        }

        globalValues = globalv.getValues();
        
        infoOut.println("Variables = {\n"+globalValues+"\n}");

        infoOut.println("PreEvaluating Vertexs");

        PreNetEvaluation[] preEval = new PreNetEvaluation[nets.length];

        for (int k = nets.length - 1; k >= 0; k--) {
            infoOut.println("Net " + k);
            preEval[k] = new PreNetEvaluation(nets[k].getVertice().length);

            for (int i = 0; i < nets[k].getVertice().length; i++) {
                long actualTime = System.currentTimeMillis();
                Vertex vertex = nets[k].getVertice()[i];

                PreEvalStrategy preEvalStrategy = new PreEvalStrategy();

                preEvalStrategy.go(vertex.getLabel().get_hooks(), globalValues);
                preEval[k].vertexs += preEvalStrategy.getTotal();
                infoOut.println("\t Vertex " + i + ", time " +
                    (long) (System.currentTimeMillis() - actualTime) +
                    ": vertexs " + preEvalStrategy.getTotal());
            }
        }

        infoOut.println("Calculating reference tables");

        for (int k = nets.length - 1; k >= 0; k--) {
            infoOut.println("Net " + k);

            int offset = 0;

            for (int i = 0; i < nets[k].getVertice().length; i++) {
                Vertex vertex = nets[k].getVertice()[i];

                long actualTime = System.currentTimeMillis();
                ReferencesStrategy vertexStrategy = new ReferencesStrategy(getVariables(
                            vertex.getLabel().get_hooks()),
                        (2 * preEval[k].vertexs) + 1);
                vertexStrategy.go(vertex.getLabel().get_hooks(), globalValues);
                preEval[k].vertexRefererences[i] = vertexStrategy.getReference();
                preEval[k].vertexRefererences[i].setOffset(offset);
                offset += vertexStrategy.getOffset();

                infoOut.println("\t Vertex " + i + ": time " +
                    (long) (System.currentTimeMillis() - actualTime) +
                    ", memory " +
                    (long) ((Runtime.getRuntime().totalMemory() -
                    Runtime.getRuntime().freeMemory()) / 1000) + "KB");
            }
        }

        infoOut.println("Genereating file");

        // print header
        netOut.println("version \"1.0\"");

        // I assume there is not declarations after instantiation
        netOut.println(Types.getTypeName(Types.TABLE_NETS) + nets.length);

        // global tables
        for (int i = 0; i < decl.get_globalTables().length; i++) {
            decl.get_globalTables()[i].pp(netOut, false);
        }

        // Expand the global Labes
        Label globalLabel = decl.getGlobalLabel();

        if (globalLabel != null) {
            //expand global labels
            Expr globalStruct = globalLabel.get_struc();

            if (globalStruct != null) {
                decl.get_globalLabel().set_struc((Expr) globalStruct.dispatch(
                        new ReplaceStructVisitor(),
                        new Object[] { globalv.getValues() }));
            }

            globalLabel.pp(netOut, false);
        }

        HashMap[] behavRef = new HashMap[nets.length];

        // for each net
        for (int k = nets.length - 1; k >= 0; k--) {
            infoOut.println("Net " + k);

            netOut.println(Types.getTypeName(Types.TYPE_NET) + k);

            Fc2Table[] tabs = nets[k].getLocalTables();

            // expand net tables
            behavRef[k] = new HashMap();

            for (int i = 0; i < tabs.length; i++) {
                if (tabs[i].getKind() == Types.TYPE_STRUCT) {
                    expandAndPrintTable(globalValues, tabs[i], new HashMap(),
                        Types.TABLE_STRUCTS);

                    continue;
                }

                if (tabs[i].getKind() == Types.TYPE_BEHAV) {
                    expandAndPrintTable(globalValues, tabs[i], behavRef[k],
                        Types.TABLE_BEHAVS);

                    continue;
                }

                tabs[i].pp(netOut, false);
            }

            //                        expand net labels
            List hooks = nets[k].getLabel().get_hooks();
            Expr struc = nets[k].getLabel().get_struc();
            Expr behav = nets[k].getLabel().get_behav();
            Expr logic = nets[k].getLabel().get_logic();

            for (int i = 0; (hooks != null) && (i < hooks.size()); i++) {
                netOut.println(Types.getTypeName(Types.TYPE_HOOK) +
                    hooks.get(i));
            }

            HashMap netReferences = new HashMap();
            int vectorSize = 0;

            if (struc != null) {
                netOut.print(Types.getTypeName(Types.TYPE_STRUCT) + " ");
                vectorSize = expandStruct(globalValues, struc, netReferences);
            }

            if (behav != null) {
                BehavVisitor behavVisitor = new BehavVisitor(netOut, infoOut);
                netOut.print(Types.getTypeName(Types.TYPE_BEHAV));
                behav.dispatch(behavVisitor,
                    new Object[] { globalValues, behavRef[k] });
                netOut.println();
            }

            if (logic != null) {
                netOut.println(Types.getTypeName(Types.TYPE_LOGIC) + logic);
            }

            int offset = 0;
            netOut.println(Types.getTypeName(Types.TABLE_VERTICE) +
                preEval[k].vertexs);

            for (int i = 0; i < nets[k].getVertice().length; i++) {
                long actualTime = System.currentTimeMillis();
                Vertex vertex = nets[k].getVertice()[i];
                VertexExpandPrintStrategy vertexExpandPrintStrategy = new VertexExpandPrintStrategy(vertex,
                        offset, netOut, infoOut, behavRef[k],
                        preEval[k].vertexRefererences, vectorSize,
                        netReferences, behavRef);
                vertexExpandPrintStrategy.go(vertex.getLabel().get_hooks(),
                    globalValues);
                offset = vertexExpandPrintStrategy.getOffset();
                infoOut.println("\t Vertex " + i + ": time " +
                    (long) (System.currentTimeMillis() - actualTime) +
                    ", edges " + vertexExpandPrintStrategy.getEdges());
            }
        }
    }

    public static void main(String[] args) throws Exception {
        CmdLineParser parser = new CmdLineParser();
        CmdLineParser.Option file = parser.addStringOption('o', "file");
        CmdLineParser.Option def = parser.addStringOption('d', "definition");
        CmdLineParser.Option verbose = parser.addBooleanOption('v',"verbose");

        try {
            parser.parse(args);
        } catch (CmdLineParser.OptionException e) {
            System.err.println("use <java_command> " +
                FC2Instantiate.class.getName() +
                " [-o <net_file_output>] -d <definicion.fc2> [-v] [<instantiations.fc2>]");
            System.err.println(e.getMessage());
            System.exit(2);
        }

        String fileName = (String) parser.getOptionValue(file);
        String defName = (String) parser.getOptionValue(def);
        Boolean verb = (Boolean)parser.getOptionValue(verbose);
        
        PrintStream netOut;
        if (fileName == null) {
        	netOut = System.out;
        } else {
        	netOut = new PrintStream(new FileOutputStream(fileName));
        }

		PrintStream infoOut;
		if (verb == null) {
			infoOut = new PrintStream(new FC2Instantiate.NullStream());
		} else {
			infoOut = System.out;
		}

        args = parser.getRemainingArgs();

        if ((args.length < 1) || (defName == null)) {
            System.err.println("use <java_command> " +
                FC2Instantiate.class.getName() +
                " -o <net_file_output> -d <definicion.fc2> [-v] [<instantiations.fc2>]+");
            System.exit(-1);
        }
        
        Fc2Parser decl = new Fc2Parser(new FileReader(defName));

        Net[] inst = new Net[args.length];

        for (int i = 0; i < args.length; i++) {
            Fc2Parser fileInst = new Fc2Parser(new FileReader(args[i]));
            fileInst.parse();
            inst[i] = fileInst.get_abs_tree();
        }

        decl.parse();

        FC2Instantiate param = new FC2Instantiate(decl.get_abs_tree(), inst,
                netOut, infoOut);

        param.expand();
    }
    
    static class NullStream extends OutputStream {

		/* (non-Javadoc)
		 * @see java.io.OutputStream#write(int)
		 */
		public void write(int b) throws IOException {
			// TODO Auto-generated method stub
			
		}
    	
    }
}
