/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.util.List;

import fr.inria.oasis.fc2.parser.Bquote;
import fr.inria.oasis.fc2.parser.ComposantVisitor;
import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.Fc2String;
import fr.inria.oasis.fc2.parser.GlobalRef;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Types;
import fr.inria.oasis.fc2.parser.Visitor;


/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class MathSolverVisitor implements Visitor {
	public static final SetMathSolverVisitor setMathSolverVisitor = new SetMathSolverVisitor();
    public Object visit(OpExpr exp, Object[] params) {
        Mapping values = (Mapping) params[0];

        if ((exp.getType() == Types.OPERATOR_CONSTANT) &&
                !Types.isPredefined(exp)) {
            Object idx = values.get(exp.getOp());

            if (idx == null) {
                Utils.reportError("The variable " + exp.getOp() +
                    " was not given in the instantiation");

                return (null);
            }

            return (idx);
        }

        /**** BOLEAN OPERATORS ****/
        if ("!".equals(exp.getOp())) {
            Boolean result = (Boolean) exp.getSons()[0].dispatch(this,
                    new Object[] { values });

            return (new Boolean(!result.booleanValue()));
        }

        if ("greaterThan".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The  greaterThan operator receives two integer parameters");

                return (null);
            }

            try {
                int hg = ((Integer) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values })).intValue();
                int mg = ((Integer) ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values })).intValue();

                return (new Boolean(hg > mg));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The  greaterThan operator receives two integer parameters");
                e.printStackTrace();

                return (null);
            }
        }

        if ("lessThan".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The  lessThan operator receives two integer parameters");

                return (null);
            }

            try {
                int hg = ((Integer) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values })).intValue();
                int mg = ((Integer) ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values })).intValue();

                return (new Boolean(hg < mg));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The  lessThan operator receives two integer parameters");
                e.printStackTrace();

                return (null);
            }
        }

        if ("equals".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The equals operator receives two parameters");

                return (null);
            }

            try {
                Object hg = ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values });
                Object mg = ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values });

                return (new Boolean(hg.equals(mg)));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The equals operator receives two parameters");
                e.printStackTrace();

                return (null);
            }
        }

        /**** ARITHMETIC OPERATORS ****/
        if ("+".equals(exp.getOp())) {
            Integer left = (Integer) exp.getSons()[0].dispatch(this,
                    new Object[] { values });
            Integer right = (Integer) exp.getSons()[1].dispatch(this,
                    new Object[] { values });

            return (new Integer(left.intValue() + right.intValue()));
        }

        if ("^".equals(exp.getOp())) {
            Integer left = (Integer) exp.getSons()[0].dispatch(this,
                    new Object[] { values });
            Integer right = (Integer) exp.getSons()[1].dispatch(this,
                    new Object[] { values });

            return (new Integer((int) Math.pow(left.intValue(), right.intValue())));
        }

        if ("-".equals(exp.getOp())) {
            Integer left = (Integer) exp.getSons()[0].dispatch(this,
                    new Object[] { values });
            Integer right = (Integer) exp.getSons()[1].dispatch(this,
                    new Object[] { values });

            return (new Integer(left.intValue() - right.intValue()));
        }

        if ("x".equals(exp.getOp())) {
            Integer left = (Integer) exp.getSons()[0].dispatch(this,
                    new Object[] { values });
            Integer right = (Integer) exp.getSons()[1].dispatch(this,
                    new Object[] { values });

            return (new Integer(left.intValue() * right.intValue()));
        }

        if ("/".equals(exp.getOp())) {
            Integer left = (Integer) exp.getSons()[0].dispatch(this,
                    new Object[] { values });
            Integer right = (Integer) exp.getSons()[1].dispatch(this,
                    new Object[] { values });

            return (new Integer(left.intValue() / right.intValue()));
        }

        if ("mod".equals(exp.getOp())) {
            Integer left = (Integer) exp.getSons()[0].dispatch(this,
                    new Object[] { values });
            Integer right = (Integer) exp.getSons()[1].dispatch(this,
                    new Object[] { values });

            return (new Integer(left.intValue() % right.intValue()));
        }

        /*** SET OPERATORS ****/
        if ("size".equals(exp.getOp())) {
            try {
                ResultMathStrategy resultMathStrategy = new ResultMathStrategy(null,
                        null);

                ComposantVisitor compv2 = new ComposantVisitor();
                exp.getSons()[0].dispatch(compv2, null);

                List composants2 = compv2.getComposants();
                resultMathStrategy.go(composants2, values);

                int size = resultMathStrategy.getResult().size();

                return (new Integer(size));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The size operator receives a set as parameter");
            }
        }

        if ("getElementAt".equals(exp.getOp())) {
            try {
                ComposantVisitor compv = new ComposantVisitor();
                exp.getSons()[0].dispatch(compv, null);

                List composants = compv.getComposants();

                if (composants.size() != 2) {
                    Utils.reportError(
                        "The operator getElementAt operator receives a Set and an integer as parameters in " +
                        exp);

                    return (null);
                }

                ResultMathStrategy resultMathStrategy = new ResultMathStrategy(null,
                        null);

                ComposantVisitor compv2 = new ComposantVisitor();
                ((Expr) composants.get(0)).dispatch(compv2, null);

                List composants2 = compv2.getComposants();
                resultMathStrategy.go(composants2, values);

                List set = resultMathStrategy.getResult();

                Integer pos = (Integer) ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values });

                try {
                    return (set.get(pos.intValue()));
                } catch (IndexOutOfBoundsException e) {
                    Utils.reportError(
                        "There is not an element at the position " + pos +
                        " for the operator getElementAt in " + exp);

                    return (null);
                }
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The operator getElementAt operator receives a Set and an integer as parameters in " +
                    exp);

                return (null);
            }
        }

        if ("getPositionOf".equals(exp.getOp())) {
            try {
                ComposantVisitor compv = new ComposantVisitor();
                exp.getSons()[0].dispatch(compv, null);

                List composants = compv.getComposants();

                if (composants.size() != 2) {
                    Utils.reportError(
                        "The operator getElementAt operator receives a Set and an integer as parameters in " +
                        exp);

                    return (null);
                }

                ResultMathStrategy resultMathStrategy = new ResultMathStrategy(null,
                        null);

                ComposantVisitor compv2 = new ComposantVisitor();
                ((Expr) composants.get(0)).dispatch(compv2, null);

                List composants2 = compv2.getComposants();
                resultMathStrategy.go(composants2, values);

                List set = resultMathStrategy.getResult();

                Object o = ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values });

                int result = set.indexOf(o);

                if (result < 0) {
                    Utils.reportError(
                        "There is not an element" + o +
                        " in the set for the operator getPositionOf in " + exp);

                    return (null);
                }

                return (new Integer(result));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The operator getPositionOf operator receives a Set and an integer as parameters in " +
                    exp);

                return (null);
            }
        }

        /**** QUEUE OPERATORS ****/
        if ("fullQueue".equals(exp.getOp())) {
            try {
                Queue queue = (Queue) exp.getSons()[0].dispatch(this,
                        new Object[] { values });

                return (new Boolean(queue.isFull()));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The fullQueue operator receives a queue as parameter");

                return (null);
            }
        }

        if ("emptyQueue".equals(exp.getOp())) {
            try {
                Queue queue = (Queue) exp.getSons()[0].dispatch(this,
                        new Object[] { values });

                return (new Boolean(queue.isEmpty()));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The emptyQueue operator receives a queue as parameter");

                return (null);
            }
        }

        if ("emptyQueueFilter".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The operator emptyQueueFilter operator receives a queue and a List");

                return (null);
            }

            try {
                Queue queue = (Queue) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values });
                List elements = (List) ((Expr) composants.get(1)).dispatch(setMathSolverVisitor,
                        new Object[] { values });

                return (new Boolean(queue.isEmpty(elements)));
            } catch (ClassCastException e) {
                Utils.reportError(
                "The operator emptyQueueFilter operator receives a queue and a List");

                return (null);
            }
        }

        if ("removeFirstFilter".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The operator removeFirstFilter operator receives a queue and a List");

                return (null);
            }

            try {
                Queue queue = (Queue) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values });
                List elements = (List) ((Expr) composants.get(1)).dispatch(setMathSolverVisitor,
                        new Object[] { values });

                return (queue.removeFirst(elements));
            } catch (ClassCastException e) {
                Utils.reportError(
                "The operator removeFirstFilter operator receives a queue and a List");

                return (null);
            }
        }

        if ("getFirstFilter".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The operator getFirstFilter operator receives a queue and a List");

                return (null);
            }

            try {
                Queue queue = (Queue) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values });
                List elements = (List) ((Expr) composants.get(1)).dispatch(setMathSolverVisitor,
                        new Object[] { values });

                return (queue.getFirst(elements));
            } catch (ClassCastException e) {
                Utils.reportError(
                "The operator getFirstFilter operator receives a queue and a List");

                return (null);
            }
        }

        if ("putQueue".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The operator putQueue operator receives a queue and a value as parameters");

                return (null);
            }

            try {
                Queue queue = (Queue) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values });
                Object call = (Object) ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values });

                return (queue.putQueue(call));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The operator putQueue receives a queue and a value as parameters");

                return (null);
            }
        }

        if ("hasElement".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The operator hasElement operator receives a queue and a value as parameters");

                return (null);
            }

            try {
                Queue queue = (Queue) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values });
                Object call = (Object) ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values });

                return (new Boolean(queue.hasElement(call)));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The operator putQueue receives a queue and a value as parameters");

                return (null);
            }
        }

        if ("removeElement".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[0].dispatch(compv, null);

            List composants = compv.getComposants();

            if (composants.size() != 2) {
                Utils.reportError(
                    "The operator removeElement operator receives a queue and a value as parameters");

                return (null);
            }

            try {
                Queue queue = (Queue) ((Expr) composants.get(0)).dispatch(this,
                        new Object[] { values });
                Object call = (Object) ((Expr) composants.get(1)).dispatch(this,
                        new Object[] { values });

                return (queue.removeElement(call));
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The operator removeElement receives a queue and a value as parameters");

                return (null);
            }
        }

        if ("getFirst".equals(exp.getOp())) {
            try {
                Queue queue = (Queue) exp.getSons()[0].dispatch(this,
                        new Object[] { values });

                return (queue.getFirst());
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The getFirst operator receives a queue as parameter");

                return (null);
            }
        }

        if ("removeFirst".equals(exp.getOp())) {
            try {
                Queue queue = (Queue) exp.getSons()[0].dispatch(this,
                        new Object[] { values });

                return (queue.removeFirst());
            } catch (ClassCastException e) {
                Utils.reportError(
                    "The getFirst operator receives a queue as parameter");

                return (null);
            }
        }

        if ("eval".equals(exp.getOp())) {
            Integer valRef = (Integer) exp.getSons()[0].dispatch(this,
                    new Object[] { values });

            if (exp.getSons()[0] instanceof OpExpr) {
                String variable = ((OpExpr) exp.getSons()[0]).getOp();
                List valArray = (List) values.get(variable);

                if (valArray == null) {
                    Utils.reportError("The variable " + variable +
                        " has not assigned values");

                    return null;
                }

                try {
                    Integer val = (Integer) valArray.get(valRef.intValue());

                    return (val);
                } catch (ClassCastException e) {
                    Utils.reportError(
                        "The math operation should be in between integers. In " +
                        exp.getOp() + " for variable " + variable + " = " +
                        valArray.get(valRef.intValue()).toString());
                }
            }

            return (exp.getSons()[0].dispatch(this, new Object[] { values }));
        }

        Utils.reportError("The operation " + exp.getOp() +
            " is not supported in " + exp);

        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        return exp.getValue().substring(1, exp.getValue().length() - 1);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
        // TODO Auto-generated method stub
        return new Integer(exp.getValue());
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
        // TODO Auto-generated method stub
        return new Integer(exp.getValue());
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
        // TODO Auto-generated method stub
        return new Integer(exp.getValue());
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
        return new Integer(exp.getValue());
    }
}
