/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.List;

import fr.inria.oasis.fc2.parser.Bquote;
import fr.inria.oasis.fc2.parser.ComposantVisitor;
import fr.inria.oasis.fc2.parser.Fc2String;
import fr.inria.oasis.fc2.parser.GlobalRef;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Visitor;


/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class NetVisitor implements Visitor {
    protected static final MathSolverVisitor mathSolverVisitor = new MathSolverVisitor();
    protected boolean started = false;
    protected HashMap references;
    int offset;
    protected StrategyTableAbstractImplementation strategyTableAbstractImplementation;
    protected PrintStream netOut;

    public NetVisitor(HashMap references, PrintStream netOut,
        PrintStream infoOut) {
        offset = 0;
        strategyTableAbstractImplementation = new StructStrategy(netOut, infoOut);
        this.netOut = netOut;

        if (references == null) {
            this.references = new HashMap();
        } else {
            this.references = references;
        }
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.OpExpr, java.lang.Object[])
     */
    public Object visit(OpExpr exp, Object[] params) {
        Mapping symbolTable = (Mapping) params[0];

        // TODO Auto-generated method stub
        if ("&".equals(exp.getOp())) {
            ComposantVisitor compv = new ComposantVisitor();
            exp.getSons()[1].dispatch(compv, null);

            List composants = compv.getComposants();

            StrategyTableAbstractImplementation strategy = strategyTableAbstractImplementation.make(null,
                    offset);
            strategy.go(composants, symbolTable);

            for (int i = 0; i < strategy.getOffset(); i++) {
                if (started) {
                    netOut.print("," + exp.getSons()[0]);
                } else {
                    netOut.print(exp.getSons()[0]);
                    started = true;
                }
            }

            references.put(exp.getSons()[0].dispatch(mathSolverVisitor, params),
                strategy.getReference());
            offset += strategy.getOffset();

            return (null);
        }

        Utils.reportError("The expression " + exp +
            " is not a valid net reference");

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        // TODO Auto-generated method stub
        Utils.reportError("Error in " + exp.toString() +
            ". The net reference should be of type Reference");

        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
        Mapping map = (Mapping) params[0];

        StrategyTableAbstractImplementation tableStrategy = strategyTableAbstractImplementation.make(exp,
                offset);
        tableStrategy.go(null, map);
        offset += tableStrategy.getOffset();

        references.put(exp.dispatch(mathSolverVisitor, params),
            tableStrategy.getReference());

        if (started) {
            netOut.print("," + exp);
        } else {
            netOut.print(exp);
            started = true;
        }

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
        Mapping map = (Mapping) params[0];

        StrategyTableAbstractImplementation tableStrategy = strategyTableAbstractImplementation.make(exp,
                offset);
        tableStrategy.go(null, map);
        offset += tableStrategy.getOffset();

        references.put(exp.dispatch(mathSolverVisitor, params),
            tableStrategy.getReference());

        if (started) {
            netOut.print("," + exp);
        } else {
            netOut.print(exp);
            started = true;
        }

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
        // TODO Auto-generated method stub
        Utils.reportError("Error in " + exp.toString() +
            ". The net reference should be of type Reference");

        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
        // TODO Auto-generated method stub
        Utils.reportError("Error in " + exp.toString() +
            ". The net reference should be of type Reference");

        return null;
    }

    /**
     * @return
     */
    public int getOffset() {
        return offset;
    }

    /**
     * @return
     */
    public HashMap getReferences() {
        return references;
    }

}
