/* 
 * ################################################################
 * 
 * FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
 * 
 * Copyright (C) 2004 INRIA
 * Contact: vercors@sophia.inria.fr
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *  
 *  Initial developer(s):               Oasis Project
 *                        http://www.inria.fr/oasis
 *  Contributor(s): 
 * 
 * ################################################################
 */

package fr.inria.oasis.fc2;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author tbarros
 * 
 * This class os a more optimal representation of a Stack using 2 integers for
 * represent the state of the stack
 */
public class Queue {
	protected static int queueDepth;

	protected static List methods;

	protected int depth;

	protected int call;

	public Queue(int depth, int call) {
		this.depth = depth;
		this.call = call;
	}

	public static int getStatesNumber(int queueDepth, int methods) {
		return ((int) (Math.pow(methods, queueDepth + 1) - 1) / (methods - 1));
	}

	public boolean equals(Object o) {
		if (o == null) {
			return (false);
		}

		try {
			Queue expr = (Queue) o;

			if ((depth == expr.getDepth()) && (call == expr.getCall())) {
				return (true);
			}

			return (false);
		} catch (ClassCastException e) {
			return (false);
		}
	}

	public int hashCode() {
		return (depth << (4 + call));
	}

	public boolean isFull() {
		if (depth >= queueDepth) {
			return (true);
		}

		return (false);
	}

	public boolean isEmpty() {
		if (depth <= 0) {
			return (true);
		}

		return (false);
	}

	public Queue putQueue(Object element) {
		int method = methods.indexOf(element);

		if (method == -1) {
			Utils.reportError("The call " + element
					+ " does not was specified in the instantiation");

			return (null);
		}

		return (new Queue(depth + 1, (call * methods.size()) + method));
	}

	public boolean hasElement(Object element) {
		List queue = getPath();
		return (queue.contains(element));

	}

	public List getPath() {
		if (depth == 0) {
			return (new ArrayList());
		}

		List pre = (new Queue(depth - 1, (int) call / methods.size()))
				.getPath();
		pre.add(methods.get(call % methods.size()));

		return (pre);
	}

	public Queue getQueue(List calls) {
		Queue queue = this;

		for (int i = 0; i < calls.size(); i++)
			queue = queue.putQueue(calls.get(i));

		return (queue);
	}

	public Object getFirst() {
		return (getPath().get(0));
	}

	public Object getFirst(List filter) {
		for (Iterator it = getPath().iterator(); it.hasNext();) {
			Object o = it.next();
			if (filter.contains(o)) {
				return (o);
			}
		}
		return (null);
	}

	public Queue removeFirst(List filter) {
		List path = getPath();

		for (Iterator it = path.iterator(); it.hasNext();) {
			Object o = it.next();
			if (filter.contains(o)) {
				it.remove();
				break;
			}
		}
		return ((new Queue(0, 0)).getQueue(path));
	}

	public boolean isEmpty(List filter) {
		if (getFirst(filter) == null) {
			return (true);
		}
		return (false);
	}

	public Queue removeFirst() {
		List path = getPath();
		path.remove(0);

		return ((new Queue(0, 0)).getQueue(path));
	}

	public Queue removeElement(Object element) {
		List path = getPath();
		path.remove(element);

		return ((new Queue(0, 0)).getQueue(path));
	}

	public static List instantiate(int maxDepth, List methods) {
		Queue.queueDepth = maxDepth;
		Queue.methods = methods;

		ArrayList result = new ArrayList();

		for (int h = 0; h <= queueDepth; h++) {
			for (int c = 0; c < (int) (Math.pow(methods.size(), h)); c++) {
				result.add(new Queue(h, c));
			}
			System.err.println("LISTO NIVEL " + h + "total " + result.size());
		}

		return (result);
	}

	/**
	 * @return
	 */
	public static List getMethods() {
		return methods;
	}

	/**
	 * @return
	 */
	public static int getQueueDepth() {
		return queueDepth;
	}

	/**
	 * @return
	 */
	public int getCall() {
		return call;
	}

	/**
	 * @return
	 */
	public int getDepth() {
		return depth;
	}

	/**
	 * @param list
	 */
	public static void setMethods(List list) {
		methods = list;
	}

	/**
	 * @param i
	 */
	public static void setQueueDepth(int i) {
		queueDepth = i;
	}

	/**
	 * @param i
	 */
	public void setCall(int i) {
		call = i;
	}

	/**
	 * @param i
	 */
	public void setDepth(int i) {
		depth = i;
	}
}