/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.util.HashMap;

import fr.inria.oasis.fc2.parser.Bquote;
import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.Fc2String;
import fr.inria.oasis.fc2.parser.GlobalRef;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Types;
import fr.inria.oasis.fc2.parser.Visitor;


/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ReplaceBehavEdgeVisitor implements Visitor {
    protected static final MergeInstancesVisitor mergeInstancesVisitor = new MergeInstancesVisitor();
    protected static final MathSolverVisitor mathSolverVisitor = new MathSolverVisitor();

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.OpExpr, java.lang.Object[])
     */
    public HashMap refTable;
    public Expr referenced;

    public ReplaceBehavEdgeVisitor(HashMap refTable) {
        this.refTable = refTable;
    }

    public Object visit(OpExpr exp, Object[] params) {
        Mapping values = (Mapping) params[0];

        if ("+".equals(exp.getOp())) {
            OpExpr nExpr = (OpExpr) exp.clone();
            nExpr.setSons(new Expr[] {
                    (Expr) exp.getSons()[0].dispatch(this, params),
                    (Expr) exp.getSons()[1].dispatch(this, params)
                });

            return (nExpr);
        }

        if (("?".equals(exp.getOp())) || ("!".equals(exp.getOp())) ||
                ("~".equals(exp.getOp()))) {
            OpExpr nExpr = (OpExpr) exp.clone();
            nExpr.setSons(new Expr[] {
                    (Expr) exp.getSons()[0].dispatch(this, params)
                });

            return (nExpr);
        }

        if (("<".equals(exp.getOp()))) {
            OpExpr nExpr = (OpExpr) exp.clone();
            nExpr.setSons(new Expr[] {
                    (Expr) exp.getSons()[0].dispatch(this, params),
                    exp.getSons()[1]
                });

            return (nExpr);
        }

        try {
            if ("&".equals(exp.getOp())) {
                referenced = exp.getSons()[0];

                Integer key = new Integer(((Ref) referenced).getValue());

                if (key == null) {
                    Utils.reportError("Error when calculating references in " +
                        exp +
                        ". (is the left side of the & operator a valid reference?");

                    return (null);
                }

                Reference ref = (Reference) refTable.get(key);

                if (ref == null) {
                    Utils.reportError(
                        "There is not a valid behaviour for the reference " +
                        key + " in " + exp);

                    return (null);
                }

                Object val = exp.getSons()[1].dispatch(mergeInstancesVisitor,
                        new Object[] { values });

                Integer replace = ref.getRef(val);

                if (replace == null) {
                    Utils.reportError(
                        "There is not a valid reference for the evaluation " +
                        val + " in " + exp);

                    return (null);
                }

                return (new Ref(replace.intValue()));
            }
        } catch (ClassCastException e) {
            Utils.reportError("Error when calculating references in " + exp +
                ". (is the left side of the & operator a valid reference?");

            return (null);
        }

        if ((exp.getType() == Types.OPERATOR_CONSTANT) &&
                !Types.isPredefined(exp)) {
            referenced = new Ref(((Integer) exp.dispatch(mathSolverVisitor,
                        params)).intValue());

            return (referenced.dispatch(this, params));
        }

        if ("t".equals(exp.getOp())) {
            // Tau case
            return (exp);
        }

        Utils.reportError("Unsuported operator for behaviour evaluation" +
            exp.getOp() + " in " + exp);

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        // TODO Auto-generated method stub
        referenced = exp;

        return (exp);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
        referenced = exp;

        Integer key = new Integer(exp.getValue());
        Reference ref = (Reference) refTable.get(key);

        if (ref == null) {
            Utils.reportError(
                "There is not a valid behaviour for the reference " + key +
                " in " + exp);

            return (null);
        }

        Integer replace = ref.getRef("");

        if (replace == null) {
            Utils.reportError(
                "There is not a valid reference for the evaluation in " + exp);

            return (null);
        }

        return (new Ref(replace.intValue()));
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
        // TODO Auto-generated method stub
        referenced = exp;

        Integer key = new Integer(exp.getValue());
        Reference ref = (Reference) refTable.get(key);

        if (ref == null) {
            Utils.reportError(
                "There is not a valid behaviour for the reference " + key +
                " in " + exp);

            return (null);
        }

        Integer replace = ref.getRef("");

        if (replace == null) {
            Utils.reportError(
                "There is not a valid reference for the evaluation in " + exp);

            return (null);
        }

        // TODO Auto-generated method stub
        return (new Ref(replace.intValue()));
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
        // TODO Auto-generated method stub
        referenced = exp;

        return (exp);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
        // TODO Auto-generated method stub
        referenced = exp;

        return (exp);
    }

    /**
     * @return
     */
    public Expr getReferenced() {
        return referenced;
    }
}
