/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.util.HashMap;

import fr.inria.oasis.fc2.parser.Bquote;
import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.Fc2String;
import fr.inria.oasis.fc2.parser.GlobalRef;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Visitor;


/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ReplaceNetVisitor implements Visitor {
    public static final MathSolverVisitor mathSolverVisitor = new MathSolverVisitor();

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.OpExpr, java.lang.Object[])
     */
    public HashMap netReferences;
    public int netPlace;
    public Expr behav;
    public HashMap[] allNetsBehavsTable;

    public ReplaceNetVisitor(HashMap netReferences,
        HashMap[] allNetsBehavsTable) {
        this.netReferences = netReferences;
        this.allNetsBehavsTable = allNetsBehavsTable;
    }

    public Object visit(OpExpr exp, Object[] params) {
        Mapping values = (Mapping) params[0];
        if (("?".equals(exp.getOp())) || ("!".equals(exp.getOp()))) {
            OpExpr nExpr = (OpExpr) exp.clone();
            nExpr.setSons(new Expr[] {
                    (Expr) exp.getSons()[0].dispatch(this, params)
                });

            return (null);
        }

        if ("$".equals(exp.getOp())) {
            ReplaceBehavEdgeVisitor repv = new ReplaceBehavEdgeVisitor(netReferences);
            netPlace = ((Ref) exp.getSons()[1].dispatch(repv,
                    new Object[] { values })).getValue();

            try {
                int net = ((Integer) repv.getReferenced().dispatch(mathSolverVisitor,
                        params)).intValue();

                if ((net < 0) || (net >= allNetsBehavsTable.length)) {
                    Utils.reportError(
                        "There is not a valid reference table for the net " +
                        net + " in " + exp + ". Is the net well defined?");

                    return (null);
                }

                HashMap refTable = (HashMap) allNetsBehavsTable[net];

                repv = new ReplaceBehavEdgeVisitor(refTable);
                behav = (Expr) exp.getSons()[0].dispatch(repv,
                        new Object[] { values });
            } catch (ClassCastException e) {
                Utils.reportError("There is not a valid reference table in " +
                    exp + ". Is the net well defined?");
            }

            return (null);
        }

        Utils.reportError(
            "A net reference should be specified using the operator $ (behaviour $ netref). In " +
            exp);

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        Utils.reportError(
            "A net reference should be specified using the operator $ (behaviour $ netref)");

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
        Utils.reportError(
            "A net reference should be specified using the operator $ (behaviour $ netref). In " +
            exp);

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
        Utils.reportError(
            "A net reference should be specified using the operator $ (behaviour $ netref). In " +
            exp);

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
        Utils.reportError(
            "A net reference should be specified using the operator $ (behaviour $ netref). In " +
            exp);

        return (null);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
        Utils.reportError(
            "A net reference should be specified using the operator $ (behaviour $ netref). In " +
            exp);

        return (null);
    }

    /**
     * @return
     */
    public Expr getBehav() {
        return behav;
    }

    /**
     * @return
     */
    public int getNetPlace() {
        return netPlace;
    }
}
