/* 
 * ################################################################
 * 
 * FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
 * 
 * Copyright (C) 2004 INRIA
 * Contact: vercors@sophia.inria.fr
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *  
 *  Initial developer(s):               Oasis Project
 *                        http://www.inria.fr/oasis
 *  Contributor(s): 
 * 
 * ################################################################
 */

package fr.inria.oasis.fc2;

import java.util.Iterator;
import java.util.List;

import fr.inria.oasis.fc2.parser.Bquote;
import fr.inria.oasis.fc2.parser.ComposantVisitor;
import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.Fc2String;
import fr.inria.oasis.fc2.parser.GlobalRef;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Visitor;

/**
 * @author tbarros
 * 
 * To change this generated comment go to Window>Preferences>Java>Code
 * Generation>Code and Comments
 */
public class StrategySetMathSolverVisitor implements Visitor {
	protected static final MathSolverVisitor mathSolverVisitor = new MathSolverVisitor();

	protected static final SetMathSolverVisitor setMathSolverVisitor = new SetMathSolverVisitor();

	public Object visit(OpExpr exp, Object[] params) {
		Mapping values = (Mapping) params[0];
		Object key = params[1];
		Strategy strategy = (Strategy) params[2];
		List assign = (List) params[3];

		if ("merge".equals(exp.getOp())) {
			ComposantVisitor compv = new ComposantVisitor();
			exp.getSons()[0].dispatch(compv, null);

			List composants = compv.getComposants();

			for (int i = 0; i < composants.size(); i++) {
				List elements = (List) ((Expr) composants.get(i)).dispatch(
						setMathSolverVisitor, new Object[] { values });
				for (Iterator it = elements 
						.iterator(); it.hasNext();) {
					Mapping map = new Mapping(values, key, it.next());
					strategy.go(assign, map);
				}
			}

			return null;
		}

		if ("set".equals(exp.getOp())) {
			ComposantVisitor compv = new ComposantVisitor();
			exp.getSons()[0].dispatch(compv, null);

			List composants = compv.getComposants();

			for (int i = 0; i < composants.size(); i++) {
				Mapping map = new Mapping(values, key, ((Expr) composants
						.get(i)).dispatch(mathSolverVisitor,
						new Object[] { values }));
				strategy.go(assign, map);
			}

			return null;
		}

		if ("in".equals(exp.getOp())) {
			ComposantVisitor compv = new ComposantVisitor();
			exp.getSons()[0].dispatch(compv, null);

			List composants = compv.getComposants();

			if (composants.size() != 2) {
				Utils
						.reportError("The in operator receives two integer parameters");

				return (null);
			}

			Integer init = (Integer) ((Expr) composants.get(0)).dispatch(
					mathSolverVisitor, new Object[] { values });
			Integer fin = (Integer) ((Expr) composants.get(1)).dispatch(
					mathSolverVisitor, new Object[] { values });

			for (int j = init.intValue(); j <= fin.intValue(); j++) {
				Mapping map = new Mapping(values, key, new Integer(j));
				strategy.go(assign, map);
			}

			return (null);
		}

		/** * QUEUE OPERATORS *** */
		if ("instantiateQueue".equals(exp.getOp())) {
			ComposantVisitor compv = new ComposantVisitor();
			exp.getSons()[0].dispatch(compv, null);

			List composants = compv.getComposants();

			if (composants.size() != 2) {
				Utils
						.reportError("The instantiateQueue operator receives two integer parameters");

				return (null);
			}

			try {
				int maxDepth = ((Integer) ((Expr) composants.get(0)).dispatch(
						mathSolverVisitor, new Object[] { values })).intValue();

				ResultMathStrategy resultMathStrategy = new ResultMathStrategy(
						null, null);

				ComposantVisitor compv2 = new ComposantVisitor();
				((Expr) composants.get(1)).dispatch(compv2, null);

				List composants2 = compv2.getComposants();

				resultMathStrategy.go(composants2, values);

				List methods = resultMathStrategy.getResult();

				Queue.queueDepth = maxDepth;
				Queue.methods = methods;

				for (int h = 0; h <= maxDepth; h++) {
					for (int c = 0; c < (int) (Math.pow(methods.size(), h)); c++) {
						Mapping map = new Mapping(values, key, new Queue(h, c));
						strategy.go(assign, map);
					}
				}

				return (null);
			} catch (ClassCastException e) {
				Utils
						.reportError("The instantiateQueue operator receives an integer and a set as parameters");
				e.printStackTrace();

				return (null);
			}
		}

		Object preVal = exp.dispatch(mathSolverVisitor, params);

		if (preVal instanceof List) {
			List val = (List) preVal;

			for (int i = 0; i < val.size(); i++) {
				Mapping map = new Mapping(values, key, val.get(i));
				strategy.go(assign, map);
			}

			return (null);
		}

		Mapping map = new Mapping(values, key, preVal);
		strategy.go(assign, map);

		return (null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String,
	 *      java.lang.Object[])
	 */
	public Object visit(Fc2String exp, Object[] params) {
		Mapping values = (Mapping) params[0];
		Object key = params[1];
		Strategy strategy = (Strategy) params[2];
		List assign = (List) params[3];

		Object preVal = exp.dispatch(mathSolverVisitor, params);

		Mapping map = new Mapping(values, key, preVal);
		strategy.go(assign, map);

		return (null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref,
	 *      java.lang.Object[])
	 */
	public Object visit(Ref exp, Object[] params) {
		Mapping values = (Mapping) params[0];
		Object key = params[1];
		Strategy strategy = (Strategy) params[2];
		List assign = (List) params[3];

		Object preVal = exp.dispatch(mathSolverVisitor, params);

		Mapping map = new Mapping(values, key, preVal);
		strategy.go(assign, map);

		return (null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef,
	 *      java.lang.Object[])
	 */
	public Object visit(GlobalRef exp, Object[] params) {
		Mapping values = (Mapping) params[0];
		Object key = params[1];
		Strategy strategy = (Strategy) params[2];
		List assign = (List) params[3];

		Object preVal = exp.dispatch(mathSolverVisitor, params);

		Mapping map = new Mapping(values, key, preVal);
		strategy.go(assign, map);

		return (null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote,
	 *      java.lang.Object[])
	 */
	public Object visit(Bquote exp, Object[] params) {
		Mapping values = (Mapping) params[0];
		Object key = params[1];
		Strategy strategy = (Strategy) params[2];
		List assign = (List) params[3];

		Object preVal = exp.dispatch(mathSolverVisitor, params);

		Mapping map = new Mapping(values, key, preVal);
		strategy.go(assign, map);

		return (null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle,
	 *      java.lang.Object[])
	 */
	public Object visit(Idle exp, Object[] params) {
		Mapping values = (Mapping) params[0];
		Object key = params[1];
		Strategy strategy = (Strategy) params[2];
		List assign = (List) params[3];

		Object preVal = exp.dispatch(mathSolverVisitor, params);

		Mapping map = new Mapping(values, key, preVal);
		strategy.go(assign, map);

		return (null);
	}
}