/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Types;


/**
 * @author bbarros
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public abstract class StrategyTableAbstractImplementation implements Strategy {
    public static final MathSolverVisitor mathSolverVisitor = new MathSolverVisitor();
    public static final StrategySetMathSolverVisitor strategySetMathSolverVisitor =
        new StrategySetMathSolverVisitor();
    public int i = 0;
    public ArrayList variables = new ArrayList();
	public int offset = 0;
	public Object preFix;
	public PrintStream netOut;
	public PrintStream infoOut;
	public Reference reference;



    public abstract void execute(Mapping map);
    
    public abstract StrategyTableAbstractImplementation make(Object preFix, int offset);
    
	public StrategyTableAbstractImplementation(PrintStream netOut, PrintStream infoOut) {
		this.netOut = netOut;
		this.infoOut = infoOut;
	}
	
	public StrategyTableAbstractImplementation(Object preFix, int offset, PrintStream netOut, PrintStream infoOut) {
		this.preFix = preFix;
		this.netOut = netOut;
		this.infoOut = infoOut;
		this.offset = offset;
	}


    public void go(List assign, Mapping map) {
        if (assign == null || assign.size() == 0) {
            execute(map);

            return;
        }

        List newList = new ArrayList(assign);

        Expr exp = (Expr) newList.remove(0);
        String varName = "var" + i;
        variables.add(new OpExpr(varName,null,Types.OPERATOR_CONSTANT));
        i++;

        exp.dispatch(strategySetMathSolverVisitor,
            new Object[] { map, varName, this, newList });
            
		i--;
		variables.remove(variables.size()-1);

    }
    
	public Expr getVariables() {
		if (variables.size() == 0) {
			return (null);
		}

		Expr exp = (Expr) variables.get(variables.size() - 1);

		for (int i = variables.size() - 2; i >= 0; i--) {
			exp = new OpExpr(",", new Expr[] { (Expr) variables.get(i), exp },
					Types.OPERATOR_INFIX, 30);
		}

		return exp;
	}

	/**
	 * @return
	 */
	public int getOffset() {
		return offset;
	}

	/**
	 * @return
	 */
	public Reference getReference() {
		return reference;
	}

}
