/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.io.PrintStream;

import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Types;
import fr.inria.oasis.fc2.parser.Visitor;

/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class SyncVector extends Expr {
	protected Expr[] behavs;
	protected Expr affectation;

	public SyncVector(Expr[] behavs, Expr affectation) {
		this.behavs = behavs;
		this.affectation = affectation;
	}

	public SyncVector(int size) {
		affectation = new Ref(0);
		behavs = new Expr[size];

		Expr idle = new Idle(1);

		for (int i = 0; i < size; i++)
			behavs[i] = idle;
	}

	public void setBehavAt(Expr e, int n) {
		behavs[n] = e;
	}

	public int hashCode() {
		int hashCode = 1;

		for (int i = 0; i < behavs.length; i++) {
			Object obj = behavs[i];
			hashCode = (31 * hashCode) + ((obj == null) ? 0 : obj.hashCode());
		}

		return (31 * hashCode + affectation.hashCode());
	}

	public boolean equals(Object o) {
		if (o == null) {
			return (false);
		}

		try {
			SyncVector expr = (SyncVector) o;

			if (((affectation == null)
				? (expr.getAffectation() == null)
				: affectation.equals(expr.getAffectation()))
				&& (behavs.length == expr.getBehavs().length)) {
				for (int i = 0; i < behavs.length; i++) {
					if (!behavs[i].equals(expr.getBehavs()[i])) {
						return (false);
					}
				}

				return (true);
			}

			return (false);
		} catch (ClassCastException e) {
			return (false);
		}
	}

	public Object clone() {
		Expr[] newSons = new Expr[behavs.length];

		for (int i = 0; i < newSons.length; i++)
			newSons[i] = (Expr) behavs[i].clone();

		return (new SyncVector(newSons, (Expr) this.affectation.clone()));
	}

	public void pp(PrintStream out) {
		if (behavs.length == 0) return;
		
		out.print(affectation + "<");
		out.print(behavs[0]);

		for (int i = 1; i < behavs.length; i++) {
			out.print(","+behavs[i]);
		}
	}

	public String toString() {
		String result = null;

		for (int i = behavs.length - 1; i >= 0; i--) {
			if (result == null) {
				result = behavs[i].toString();
			} else {
				result = behavs[i].toString() + "," + result;
			}
		}
		return (affectation + "<" + result);

	}

	public Object dispatch(Visitor visitor, Object[] params) {
		return (null);
	}
	public Expr toExpr() {
		Expr result = null;

		for (int i = behavs.length - 1; i >= 0; i--) {
			if (result == null) {
				result = behavs[i];
			} else {
				result =
					new OpExpr(
						",",
						new Expr[] { behavs[i], result },
						Types.OPERATOR_INFIX,
						30);
			}
		}

		result =
			new OpExpr(
				"<",
				new Expr[] { affectation, result },
				Types.OPERATOR_INFIX,
				30);

		return (result);
	}

	/**
	 * @return Expr
	 */
	public Expr getAffectation() {
		return affectation;
	}

	/**
	 * Sets the affectation.
	 * @param affectation The affectation to set
	 */
	public void setAffectation(Expr affectation) {
		this.affectation = affectation;
	}
	/**
	 * Returns the behavs.
	 * @return Expr[]
	 */
	public Expr[] getBehavs() {
		return behavs;
	}

}
