/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import fr.inria.oasis.fc2.parser.Bquote;
import fr.inria.oasis.fc2.parser.Expr;
import fr.inria.oasis.fc2.parser.Fc2String;
import fr.inria.oasis.fc2.parser.GlobalRef;
import fr.inria.oasis.fc2.parser.Idle;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Ref;
import fr.inria.oasis.fc2.parser.Visitor;



/**
 * @author tbarros
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class TargetVisitor implements Visitor {
	protected static final MergeInstancesVisitor mergeInstancesVisitor =new MergeInstancesVisitor();
	protected static final MathSolverVisitor mathSolverVisitor =new MathSolverVisitor();
	protected Reference[] vertexReferences;

public TargetVisitor(Reference[] vertexReferences) {
	this.vertexReferences = vertexReferences;
}

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.OpExpr, java.lang.Object[])
     */
    public Object visit(OpExpr exp, Object[] params) {
        // TODO Auto-generated method stub
        if ("&".equals(exp.getOp())) {
           Object valRef =  exp.getSons()[1].dispatch(mergeInstancesVisitor,
                    params);
                    
            try {
            	int vertex = ((Integer)exp.getSons()[0].dispatch(mathSolverVisitor,params)).intValue();
            	if (vertex <0 || vertex >= vertexReferences.length || vertexReferences[vertex] == null) {
            		Utils.reportError("There is not a valid reference for the vertex value "+vertex+"in "+exp);
            		return(null);
            	}
            	
            	Reference ref = vertexReferences[vertex];
            	
				Integer val = (Integer) ref.getRef(valRef);

				if (val == null) {
					Utils.reportError(
						"There is not a valid reference for a vertex in " + exp);

					return (null);
				}

				return (new Ref(val.intValue()));
            	
            } catch (ClassCastException e) {
				Utils.reportError("There is not a valid vertex reference in "+exp);
				return(null);
            }
        }

        if ("+".equals(exp.getOp())) {
			OpExpr newExp = (OpExpr) exp.clone();
            newExp.getSons()[0] = (Expr) newExp.getSons()[0].dispatch(this, params);
			newExp.getSons()[1] = (Expr) newExp.getSons()[1].dispatch(this, params);
            return(exp);
        }

        Utils.reportError("Unsuported operator " + exp.getOp() + " in " + exp);

        return null;
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Fc2String, java.lang.Object[])
     */
    public Object visit(Fc2String exp, Object[] params) {
        return (exp);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Ref, java.lang.Object[])
     */
    public Object visit(Ref exp, Object[] params) {
		return (exp);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.GlobalRef, java.lang.Object[])
     */
    public Object visit(GlobalRef exp, Object[] params) {
		return (exp);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Bquote, java.lang.Object[])
     */
    public Object visit(Bquote exp, Object[] params) {
		return (exp);
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.Visitor#visit(fr.inria.oasis.fc2.parser.Idle, java.lang.Object[])
     */
    public Object visit(Idle exp, Object[] params) {
		return (exp);
    }
}
