/* 
* ################################################################
* 
* FC2Parameterized: A Java(TM) interpreter for Parameterized FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 

package fr.inria.oasis.fc2;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.List;

import fr.inria.oasis.fc2.parser.Edge;
import fr.inria.oasis.fc2.parser.OpExpr;
import fr.inria.oasis.fc2.parser.Types;
import fr.inria.oasis.fc2.parser.Vertex;


/**
 * @author bbarros
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class VertexExpandPrintStrategy extends StrategyAbstractImplementation {
    protected Vertex vertex;
    public int offset;
    public PrintStream netOut;
    public PrintStream infoOut;
    protected HashMap behavsReferences;
    protected Reference[] vertexReferences;
    protected int vectorSize;
    protected HashMap netReferences;
    protected HashMap[] allNetsBehavsTable;
    protected int edges;

    public VertexExpandPrintStrategy(Vertex vertex, int offset,
        PrintStream netOut, PrintStream infoOut, HashMap behavsReferences,
        Reference[] vertexReferences, int vectorSize, HashMap netReferences,
        HashMap[] allNetsBehavsTable) {
        this.offset = offset;
        this.vertex = vertex;
        this.netOut = netOut;
        this.infoOut = infoOut;
        this.behavsReferences = behavsReferences;
        this.vertexReferences = vertexReferences;
        this.vectorSize = vectorSize;
        this.netReferences = netReferences;
        this.allNetsBehavsTable = allNetsBehavsTable;
        this.edges = 0;
    }

    protected void removeHooks(List hooks) {
        int i = 0;

        while (i < hooks.size()) {
            try {
                OpExpr exp = (OpExpr) hooks.get(i);

                if (("when".equals(exp.getOp())) || ("=".equals(exp.getOp()))) {
                    hooks.remove(i);

                    continue;
                }
            } catch (ClassCastException e) {
            }

            i++;
        }
    }

    /* (non-Javadoc)
     * @see fr.inria.oasis.fc2.parser.StrategyAbstractImplementation#execute(fr.inria.oasis.fc2.parser.Mapping)
     */
    public void execute(Mapping map) {
        // TODO Auto-generated method stub
        int edgeNum = 0;

        Vertex newVertex = (Vertex) vertex.clone();
        removeHooks(newVertex.getLabel().get_hooks());

        netOut.println(Types.getTypeName(Types.TYPE_VERTEX) + offset);
        offset++;
        newVertex.getLabel().pp(netOut, false);

        for (int i = 0; i < vertex.getEdges().length; i++) {
            Edge edge = vertex.getEdges()[i];
            PreEvalStrategy preEvalStrategy = new PreEvalStrategy();
            preEvalStrategy.go(edge.getLabel().get_hooks(), map);
            edgeNum += preEvalStrategy.getTotal();
        }

        this.edges += edgeNum;

        int edgeOffset = 0;

        netOut.println(Types.getTypeName(Types.TABLE_EDGES) + edgeNum);

        for (int i = 0; i < vertex.getEdges().length; i++) {
            Edge edge = vertex.getEdges()[i];

            EdgeExpandPrintStrategy edgeExpandPrintStrategy = new EdgeExpandPrintStrategy(edge,
                    edgeOffset, netOut, infoOut, behavsReferences,
                    vertexReferences, vectorSize, netReferences,
                    allNetsBehavsTable);
            edgeExpandPrintStrategy.go(edge.getLabel().get_hooks(), map);
            edgeOffset = edgeExpandPrintStrategy.getOffset();
        }
    }

    /**
     * @return
     */
    public int getOffset() {
        return offset;
    }

    /**
     * @return
     */
    public int getEdges() {
        return edges;
    }
}
